require_relative "spec_helper"

describe Gemnigma::CipherDecrypt do
  it "should be a module" do
    expect(Gemnigma::CipherDecrypt.class).to eql(Module)
  end

  it "should have a method #rotate_backward" do
    expect(Gemnigma::CipherDecrypt.methods.include? :rotate_backward).to be true
  end

  describe "#rotate_backward" do
    context "without parameters" do
      it "should raise an exception" do
        expect { Gemnigma::CipherDecrypt.rotate_backward }.to raise_error(ArgumentError)
      end
    end

    context "with parameters" do
      it "should return a hash" do
        expect(Gemnigma::CipherDecrypt.rotate_backward(3).class).to eql(Hash)
      end

      it "should return a hash with a length of 39" do
        expect(Gemnigma::CipherDecrypt.rotate_backward(3).length).to eql(39)
      end

      it "should be a reverse of CipherEncrypt #rotate_forward method" do
        expect(Gemnigma::CipherDecrypt.rotate_backward(3)).to eql(Gemnigma::CipherEncrypt.rotate_forward(3).invert)
      end

      it "should reverse a rotation if the rotation number match" do
        word = Gemnigma::CipherEncrypt.rotate_forward(3)["a"]
        expect(Gemnigma::CipherDecrypt.rotate_backward(3)[word]).to eql("a")
      end

      it "should reverse a rotation if the rotation number is an addition of the number and 39" do
        word = Gemnigma::CipherEncrypt.rotate_forward(3)["a"]
        expect(Gemnigma::CipherDecrypt.rotate_backward(3 + 39)[word]).to eql("a")
      end

      it "should reverse a rotation with rotation number plus a multiple of 39" do
        word = Gemnigma::CipherEncrypt.rotate_forward(3)["a"]
        expect(Gemnigma::CipherDecrypt.rotate_backward(3 + 78)[word]).to eql("a")
        expect(Gemnigma::CipherDecrypt.rotate_backward(3 + 117)[word]).to eql("a")
      end

      it "should not reverse a rotation if the rotation number doesn't match" do
        word = Gemnigma::CipherEncrypt.rotate_forward(3)["a"]
        expect(Gemnigma::CipherDecrypt.rotate_backward(4)[word]).not_to eql("a")
      end
    end
  end

end