require_relative "spec_helper"

describe Gemnigma::Decrypt do
  before :each do
    @dec = Gemnigma::Decrypt.new
  end

  it "should be a subclass of RotatorDecrypt" do
    expect(Gemnigma::Decrypt.superclass).to eql(Gemnigma::RotatorDecrypt)
  end

  it "should be an instance of the class Decrypt" do
    expect(@dec).to be_an_instance_of Gemnigma::Decrypt
  end

  describe "#get_cmd_args" do
    it "should take no arguments" do
      expect{ @dec.get_cmd_args(123) }.to raise_error(ArgumentError)
    end

   it "should display an error message if few arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.few_args

      expect(io).to have_received(:puts).with("Not enough arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end

    it "should display an error message if too many arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.too_much_args

      expect(io).to have_received(:puts).with("Too many arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end
  end

  describe "#true_key?" do
    it "should be a private method" do
      expect{ @dec.true_key? }.to raise_error (NoMethodError)
    end

    it "should take one argument only" do
      expect{ @dec.send(:true_key?) }.to raise_error(ArgumentError)
      expect{ @dec.send(:true_key?, "41521", "32415") }.to raise_error(ArgumentError)
    end

    it "should return true for valid keys" do
      expect(@dec.send(:true_key?, "31292")).to be true
      expect(@dec.send(:true_key?, "43556")).to be true
      expect(@dec.send(:true_key?, "41521")).to be true
      expect(@dec.send(:true_key?, "12345")).to be true
    end

    it "should return false for invalid keys" do
      expect(@dec.send(:true_key?, "abcde")).to be false
      expect(@dec.send(:true_key?, "41526a")).to be false
      expect(@dec.send(:true_key?, "4152")).to be false
      expect(@dec.send(:true_key?, "415216")).to be false
    end
  end

  describe "#true_date?" do
    it "should be a private method" do
      expect{ @dec.true_date?(2) }.to raise_error(NoMethodError)
    end

    it "should take one compulsory argument" do
      expect{ @dec.send(:true_date?) }.to raise_error(ArgumentError)
      expect{ @dec.send(:true_date?, 123456, 90919) }.to raise_error(ArgumentError)
    end

    it "should return true for valid dates" do
      expect(@dec.send(:true_date?, 11015)).to be true
      expect(@dec.send(:true_date?, 81015)).to be true
    end

    it "should return false for invalid dates" do
      expect(@dec.send(:true_date?, 110152)).to be false
      expect(@dec.send(:true_date?, "hello")).to be false
    end
  end

  describe "#Decrypt_file" do
    it "should be a private method" do
      expect{ @dec.Decrypt_file }.to raise_error (NoMethodError)
    end

    it "should take four arguments" do
      expect { @dec.send(:decrypt_file) }.to raise_error(ArgumentError)
      expect { @dec.send(:decrypt_file, "message.txt") }.to raise_error(ArgumentError)
      expect { @dec.send(:decrypt_file, "message.txt", "enc.txt") }.to raise_error(ArgumentError)
      expect { @dec.send(:decrypt_file, "message.txt", "enc.txt", 24515) }.to raise_error(ArgumentError)
    end

    it "should display a success message if file was successfully created" do
      
      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.success_message("file.txt", "94704", "81015")

      expect(io).to have_received(:puts).with("\n........................................................")
      expect(io).to have_received(:puts).with("\nCreated 'file.txt' with secret key 94704 and date 81015")
    end
  end
end