require_relative "spec_helper"

describe Gemnigma::Encrypt do
  before :each do
    @enc = Gemnigma::Encrypt.new
  end

  it "should be a subclass of RotatorEncrypt" do
    expect(Gemnigma::Encrypt.superclass).to eql(Gemnigma::RotatorEncrypt)
  end

  it "should be an instance of the class Encrypt" do
    expect(@enc).to be_an_instance_of Gemnigma::Encrypt
  end

  describe "#get_cmd_args" do
    it "should take no arguments" do
      expect{ @enc.get_cmd_args(123) }.to raise_error(ArgumentError)
    end

    it "should display an error message if few arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.few_args

      expect(io).to have_received(:puts).with("Not enough arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end

    it "should display an error message if too many arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.too_much_args

      expect(io).to have_received(:puts).with("Too many arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end
  end

  describe "#encrypt_file" do
    it "should be a private method" do
      expect{ @enc.encrypt_file }.to raise_error (NoMethodError)
    end

    it "should take two arguments" do
      expect { @enc.send(:encrypt_file) }.to raise_error(ArgumentError)
      expect { @enc.send(:encrypt_file, "message.txt") }.to raise_error(ArgumentError)
      expect { @enc.send(:encrypt_file, "message.txt", "file.txt", "io.txt") }.to raise_error(ArgumentError)
    end

    it "should display a success message if file was successfully created" do
      
      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.success_message("file.txt", "94704", "81015")

      expect(io).to have_received(:puts).with("\n........................................................")
      expect(io).to have_received(:puts).with("\nCreated 'file.txt' with secret key 94704 and date 81015")
    end
  end
end