require_relative "spec_helper"

describe Gemnigma::Messages do
  before :each do
    @msg = Gemnigma::Messages.new
  end

  it "should be an instance of the Messages class" do
    expect(@msg).to be_an_instance_of Gemnigma::Messages
  end

  describe "#initialize" do
    it "should default to standard output as the default if no argument is supplied to new" do
      is_stdout = (@msg.instance_variable_get(:@io) == STDOUT)
      expect(is_stdout).to be true
    end

    it "should take whatever argument is passed into it as the output stream" do
      msg = Gemnigma::Messages.new(StringIO.new)
      expect(msg.instance_variable_get(:@io).class).to eql(StringIO)
    end
  end

  describe "#puts" do
    it "should take one compulsory argument" do
      expect{ @msg.puts }.to raise_error(ArgumentError)
      expect{ @msg.puts("hello", "world") }.to raise_error(ArgumentError)
    end

    it "should print to the console whatever is passed into it" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.puts "Hello World"
      msg.puts("This is a ridiculously long message")

      expect(io).to have_received(:puts).with("Hello World")
      expect(io).to have_received(:puts).with("This is a ridiculously long message")
    end
  end

  describe "#few_args" do
    it "should print to the console when few_args is called" do
         
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.few_args

      expect(io).to have_received(:puts).with("Not enough arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end
  end

  describe "#too_much_args" do
    it "should print to the console when too_much_args is called" do

      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.too_much_args

      expect(io).to have_received(:puts).with("Too many arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end
  end

  describe "#file_not_in_existence" do
    it "should take one compulsory argument" do
      expect{ @msg.file_not_in_existence }.to raise_error(ArgumentError)
      expect{ @msg.file_not_in_existence("file", "new_file") }.to raise_error(ArgumentError)
    end
    
    it "should print to the console when file_not_in_existence is called" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.file_not_in_existence("file.txt")

      expect(io).to have_received(:puts).with("file.txt: File does not exist\nExiting...")
    end
  end

  describe "#file_in_existence" do
    it "should take two compulsory argument" do
      expect{ @msg.file_in_existence }.to raise_error(ArgumentError)
      expect{ @msg.file_in_existence("file", "new_file", "another_file") }.to raise_error(ArgumentError)
    end
    
    it "should print to the console when file_in_existence is called" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.file_in_existence("file.txt", "new_file.txt")

      expect(io).to have_received(:puts).with("file.txt exists... Checking new_file.txt")
    end
  end

  describe "#creating_new_file" do
    it "should take one compulsory argument" do
      expect{ @msg.creating_new_file }.to raise_error(ArgumentError)
      expect{ @msg.creating_new_file("file", "new_file") }.to raise_error(ArgumentError)
    end
    
    it "should print to the console when creating_new_file is called" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.creating_new_file("file.txt")

      expect(io).to have_received(:puts).with("file.txt will be created, proceeding...")
    end
  end

  describe "#overwrite_warning" do
    it "should take one compulsory argument" do
      expect{ @msg.overwrite_warning }.to raise_error(ArgumentError)
      expect{ @msg.overwrite_warning("file", "new_file") }.to raise_error(ArgumentError)
    end
    
    it "should print to the console when overwrite_warning is called" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.overwrite_warning("file.txt")

      expect(io).to have_received(:puts).with( "file.txt exists already, do you want to overwrite it? (Y) or (N): ")
    end
  end

  describe "#wrong_date" do
     it "should take one compulsory argument" do
      expect{ @msg.wrong_date }.to raise_error(ArgumentError)
      expect{ @msg.wrong_date("12345", "09190002") }.to raise_error(ArgumentError)
    end

    it "should print to the console when wrong_date is called" do

      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.wrong_date("abcde")

      expect(io).to have_received(:puts).with("abcde doesn't match the specified date format\nExiting...")
    end
  end

  describe "#success_message" do
    it "should take three compulsory arguments" do
      expect{ @msg.success_message }.to raise_error(ArgumentError)
      expect{ @msg.success_message("file.txt", "new.txt") }.to raise_error(ArgumentError)
      expect{ @msg.success_message("file.txt", "new.txt", "810", "105") }.to raise_error(ArgumentError)
    end

    it "should print to the console when success_message is called" do
      
      io = double("io", puts: nil)
      msg = Gemnigma::Messages.new(io)
      msg.success_message("new.txt", "94704", "81015")

      expect(io).to have_received(:puts).with("\n........................................................")
      expect(io).to have_received(:puts).with("\nCreated 'new.txt' with secret key 94704 and date 81015")
    end
  end
end