require_relative "spec_helper"

describe Gemnigma::Offset do
  before :each do 
    @offset = Gemnigma::Offset.new
  end

  it "should return Offset as it's class name" do
    expect(@offset).to be_an_instance_of Gemnigma::Offset
  end

  describe "#date" do
    it "should return the current date" do
      expect(@offset.date.asctime).to eql(Time.now.asctime)
    end
  end

  describe "#formatted_date" do
    it "should be a private method" do
      expect { @offset.formatted_date }.to raise_error(NoMethodError)
    end

    it "should return a formatted date in the format ddmmyy" do
      expect(@offset.send(:formatted_date).to_s).to match(/\d\d\d\d\d/)
    end
  end

  describe "#squared_date" do
    it "should be a private method" do
      expect { @offset.squared_date }.to raise_error(NoMethodError)
    end

    it "should return the square of the formatted_date" do
      square = @offset.send(:formatted_date) ** 2
      expect(@offset.send(:squared_date)).to eql(square)
    end
  end

  describe "#offset" do
    it "should return an array" do
      expect(@offset.offset.class).to eql(Array)
    end

    it "should return an array with a length of 4" do
      expect(@offset.offset.length).to eql(4)
    end

    it "should return an array of integers" do
      expect(@offset.offset.all? {|offset| offset.class == Fixnum}).to be true
    end
  end

  describe "#full_date" do
    it "should return an integer" do
      expect(@offset.full_date.is_a? Integer).to be true
    end

    it "should return an integer corresponding to date the offset is based upon" do
      expect(@offset.full_date.to_s).to match(/\d\d\d\d\d/)
    end
  end
end