require_relative "spec_helper"

describe Gemnigma::ValidateFileArg do
  it "should be a module" do
    expect(Gemnigma::ValidateFileArg.class).to eql(Module)
  end

  it "should have a method validate" do
    expect(Gemnigma::ValidateFileArg.instance_methods.include? :validate).to be true
  end

  describe "#validate" do
    it "should display an error message if input file does not exist" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.file_not_in_existence("non-existent.txt")

      expect(io).to have_received(:puts).with("non-existent.txt: File does not exist\nExiting...")
    end

    it "should display a success message if file exists" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.file_in_existence("message.txt", "new_file.txt")

      expect(io).to have_received(:puts).with("message.txt exists... Checking new_file.txt")
    end

    it "should display a informing a user that a file is about to be created" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.creating_new_file("new_file.txt")

      expect(io).to have_received(:puts).with("new_file.txt will be created, proceeding...")
    end

    it "should display a message alerting a user that a file is already in existence" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.overwrite_warning("new_file.txt")

      expect(io).to have_received(:puts).with("new_file.txt exists already, do you want to overwrite it? (Y) or (N): ")
    end
  end
end